@@ -1,9 +1,9 @@
 /* vim: set sw=8: */
 
 /*
- * excel-xml-read.c : Read MS Excel's xml
+ * excel-xml-read.c : Read MS Excel 2003 SpreadsheetML
  *
- * Copyright (C) 2003-2005 Jody Goldberg (jody@gnome.org)
+ * Copyright (C) 2003-2008 Jody Goldberg (jody@gnome.org)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -41,13 +41,12 @@
 #include "command-context.h"
 #include "workbook-view.h"
 #include "workbook.h"
-#include <goffice/app/error-info.h>
-#include <goffice/app/io-context.h>
-#include <goffice/app/go-plugin.h>
-#include <goffice/utils/datetime.h>
+#include "xml-io.h"
+#include <goffice/goffice.h>
 
 #include <gsf/gsf-libxml.h>
 #include <gsf/gsf-input.h>
+#include <gsf/gsf-utils.h>
 #include <gmodule.h>
 #include <glib/gi18n-lib.h>
 #include <stdlib.h>
@@ -61,7 +60,7 @@
 
 typedef struct {
 	GnumericXMLVersion version;
-	IOContext	*context;	/* The IOcontext managing things */
+	GOIOContext	*context;	/* The IOcontext managing things */
 	WorkbookView	*wb_view;	/* View for the new workbook */
 	Workbook	*wb;		/* The new workbook */
 	Sheet		*sheet;		/* The current sheet */
@@ -113,7 +112,7 @@ xl_xml_warning (GsfXMLIn *xin, char const *fmt, ...)
 		msg = tmp;
 	}
 
-	gnm_io_warning (state->context, "%s", msg);
+	go_io_warning (state->context, "%s", msg);
 	g_warning ("%s", msg);
 	g_free (msg);
 
@@ -128,7 +127,7 @@ unknown_attr (GsfXMLIn *xin,
 	g_return_if_fail (attrs != NULL);
 
 	if (state->version == GNM_XML_LATEST)
-		gnm_io_warning (state->context,
+		go_io_warning (state->context,
 			_("Unexpected attribute %s::%s == '%s'."),
 			name, attrs[0], attrs[1]);
 }
@@ -318,11 +317,11 @@ xl_xml_col_start (GsfXMLIn *xin, xmlChar const **attrs)
 		r.start.col = state->pos.col;
 		r.end.col   = state->pos.col + span - 1;
 		r.start.row = 0;
-		r.end.row  = SHEET_MAX_ROWS - 1;
+		r.end.row  = gnm_sheet_get_max_rows (state->sheet) - 1;
 		gnm_style_ref (style);
 		sheet_style_set_range (state->sheet, &r, style);
 	}
-	if (width >= 0.)
+	if (width > 0.)
 		for (tmp = 0 ; tmp < span ; tmp++)
 			sheet_col_set_size_pts (state->sheet,
 				state->pos.col + tmp, width, !auto_fit);
@@ -373,7 +372,7 @@ xl_xml_row_start (GsfXMLIn *xin, xmlChar const **attrs)
 		r.start.row = state->pos.row;
 		r.end.row   = state->pos.row + span - 1;
 		r.start.col = 0;
-		r.end.col  = SHEET_MAX_COLS - 1;
+		r.end.col  = gnm_sheet_get_max_cols (state->sheet) - 1;
 		gnm_style_ref (style);
 		sheet_style_set_range (state->sheet, &r, style);
 	}
@@ -476,7 +475,7 @@ xl_xml_data_end (GsfXMLIn *xin, G_GNUC_UNUSED GsfXMLBlob *blob)
 			g_date_clear (&date, 1);
 			g_date_set_dmy (&date, d, mo, y);
 			if (g_date_valid (&date)) {
-				unsigned d_serial = datetime_g_to_serial (&date,
+				unsigned d_serial = go_date_g_to_serial (&date,
 					workbook_date_conv (state->wb));
 				v = value_new_float (d_serial + h/24. + mi/(24.*60.) + s/(24.*60.*60.));
 			} else
@@ -703,20 +702,51 @@ xl_xml_num_interior (GsfXMLIn *xin, xmlChar const **attrs)
 static void
 xl_xml_num_fmt (GsfXMLIn *xin, xmlChar const **attrs)
 {
+	static struct {
+		char const *name;
+		GOFormatMagic id;
+	} named_magic_formats [] = {
+		{ "General Date",        GO_FORMAT_MAGIC_SHORT_DATETIME },
+		{ "Long Date",           GO_FORMAT_MAGIC_LONG_DATE },
+		{ "Medium Date",         GO_FORMAT_MAGIC_MEDIUM_DATE },
+		{ "Short Date",          GO_FORMAT_MAGIC_SHORT_DATE },
+		{ "Long Time",           GO_FORMAT_MAGIC_LONG_TIME },
+		{ "Medium Time",         GO_FORMAT_MAGIC_MEDIUM_TIME },
+		{ "Short Time",          GO_FORMAT_MAGIC_SHORT_TIME },
+		{ NULL, 0 }
+	};
+	static struct {
+		char const *name;
+		char const *format;
+	} named_formats [] = {
+		{ "General Number",     "General" },
+		{ "Currency",       	"$#,##0.00_);[Red](#,##0.00)" },
+		{ "Euro Currency",     	"[$EUR-2]#,##0.00_);[Red](#,##0.00)" },
+		{ "Fixed",              "0.00" },
+		{ "Standard",           "#,##0.00" },	/* number, 2dig, +sep */
+		{ "Percent",            "0.00%" },	/* std percent */
+		{ "Scientific",         "0.00E+00" },	/* std scientific */
+		{ "Yes/No",		"\"Yes\";\"Yes\";\"No\"" },
+		{ "True/False",		"\"True\";\"True\";\"False\"" },
+		{ "On/Off",		"\"On\";\"On\";\"Off\"" },
+		{ NULL, 0 }
+	};
 	ExcelXMLReadState *state = (ExcelXMLReadState *)xin->user_state;
 	for (; attrs != NULL && attrs[0] && attrs[1] ; attrs += 2)
 		if (gsf_xml_in_namecmp (xin, attrs[0], XL_NS_SS, "Format")) {
 			GOFormat *fmt = NULL;
-			if (!strcmp (attrs[1], "Percent"))
-				fmt = go_format_default_percentage ();
-			else if (!strcmp (attrs[1], "Short Time"))
-				fmt = go_format_default_time ();
+			int i;
 
-			if (NULL != fmt)
-				go_format_ref (fmt);
-			else if (!strcmp (attrs[1], "Fixed"))
-				fmt = go_format_new_from_XL ("0.00");
-			else
+			for (i = 0 ; named_formats[i].name ; i++)
+				if (0 == strcmp (attrs[1], named_formats[i].name))
+					fmt = go_format_new_from_XL (named_formats[i].format);
+
+			if (NULL == fmt)
+				for (i = 0 ; named_magic_formats[i].name ; i++)
+					if (0 == strcmp (attrs[1], named_magic_formats[i].name))
+						fmt = go_format_new_magic (named_magic_formats[i].id);
+
+			if (NULL == fmt)
 				fmt = go_format_new_from_XL (attrs[1]);
 			gnm_style_set_format (state->style, fmt);
 			go_format_unref (fmt);
@@ -795,7 +825,8 @@ xl_xml_sheet_start (GsfXMLIn *xin, xmlChar const **attrs)
 		g_return_if_fail (state->sheet == NULL);
 		state->sheet =  workbook_sheet_by_name (state->wb, name);
 		if (state->sheet == NULL) {
-			state->sheet = sheet_new (state->wb, name);
+		  state->sheet = sheet_new (state->wb, name,
+					    256, 65536);  /* FIXME */
 			workbook_sheet_attach (state->wb, state->sheet);
 		}
 
@@ -899,6 +930,9 @@ GSF_XML_IN_NODE_FULL (START, WORKBOOK, XL_NS_SS, "Workbook", GSF_XML_NO_CONTENT,
     GSF_XML_IN_NODE (DOC_PROP, PROP_LAST_SAVED,	 XL_NS_O, "LastSaved",  GSF_XML_CONTENT, NULL, NULL),
     GSF_XML_IN_NODE (DOC_PROP, PROP_COMPANY,	 XL_NS_O, "Company",    GSF_XML_CONTENT, NULL, NULL),
     GSF_XML_IN_NODE (DOC_PROP, PROP_VERSION,	 XL_NS_O, "Version",    GSF_XML_CONTENT, NULL, NULL),
+    GSF_XML_IN_NODE (DOC_PROP, PROP_TITLE,      XL_NS_O, "Title",      GSF_XML_CONTENT, NULL, NULL),
+    GSF_XML_IN_NODE (DOC_PROP, PROP_DESCRIPTION, XL_NS_O, "Description",GSF_XML_CONTENT, NULL, NULL),
+
   GSF_XML_IN_NODE (WORKBOOK, DOC_SETTINGS, XL_NS_O, "OfficeDocumentSettings", GSF_XML_NO_CONTENT, NULL, NULL),
     GSF_XML_IN_NODE (DOC_SETTINGS, DOC_COLORS, XL_NS_O, "Colors", GSF_XML_NO_CONTENT, NULL, NULL),
       GSF_XML_IN_NODE (DOC_COLORS, DOC_COLOR,  XL_NS_O, "Color", GSF_XML_NO_CONTENT, NULL, NULL),
@@ -977,11 +1011,44 @@ GSF_XML_IN_NODE_FULL (START, WORKBOOK, XL_NS_SS, "Workbook", GSF_XML_NO_CONTENT,
 };
 
 G_MODULE_EXPORT void
-excel_xml_file_open (GOFileOpener const *fo, IOContext *context,
+excel_xml_file_open (GOFileOpener const *fo, GOIOContext *context,
 		     WorkbookView *wbv, GsfInput *input);
 
+G_MODULE_EXPORT gboolean
+excel_xml_file_probe (GOFileOpener const *fo, GsfInput *input, GOFileProbeLevel pl);
+
+static gboolean
+xl_xml_probe_start_element (const xmlChar *name,
+			    G_GNUC_UNUSED const xmlChar *prefix,
+			    const xmlChar *URI,
+			    G_GNUC_UNUSED int nb_namespaces,
+			    G_GNUC_UNUSED const xmlChar **namespaces,
+			    G_GNUC_UNUSED int nb_attributes,
+			    G_GNUC_UNUSED int nb_defaulted,
+			    G_GNUC_UNUSED const xmlChar **attributes)
+{
+	/* starts with <Workbook> in namespace "schemas-microsoft-com:office:spreadsheet" */
+	return 0 == strcmp (name, "Workbook") &&
+		NULL != URI &&
+		NULL != strstr (URI, "schemas-microsoft-com:office:spreadsheet");
+}
+
+gboolean
+excel_xml_file_probe (GOFileOpener const *fo, GsfInput *input, GOFileProbeLevel pl)
+{
+	if (pl == GO_FILE_PROBE_FILE_NAME) {
+		char const *ext;
+		char const *name = gsf_input_name (input);
+		return  NULL != name &&
+			NULL != (ext = gsf_extension_pointer (name)) &&
+			0 == g_ascii_strcasecmp (ext, "xml");
+	}
+
+	return gsf_xml_probe (input, &xl_xml_probe_start_element);
+}
+
 void
-excel_xml_file_open (GOFileOpener const *fo, IOContext *io_context,
+excel_xml_file_open (GOFileOpener const *fo, GOIOContext *io_context,
 		     WorkbookView *wb_view, GsfInput *input)
 {
 	GsfXMLInDoc *doc;
@@ -999,7 +1066,7 @@ excel_xml_file_open (GOFileOpener const *fo, IOContext *io_context,
 
 	doc = gsf_xml_in_doc_new (excel_xml_dtd, content_ns);
 	if (!gsf_xml_in_doc_parse (doc, input, &state))
-		gnumeric_io_error_string (io_context, _("XML document not well formed!"));
+		go_io_error_string (io_context, _("XML document not well formed!"));
 	gsf_xml_in_doc_free (doc);
 
 	g_hash_table_destroy (state.style_hash);
