@@ -26,7 +26,7 @@
 
 GNM_PLUGIN_MODULE_HEADER;
 
-void xbase_file_open (GOFileOpener const *fo, IOContext *io_context,
+void xbase_file_open (GOFileOpener const *fo, GOIOContext *io_context,
                       WorkbookView *wb_view, GsfInput *input);
 
 
@@ -93,7 +93,7 @@ xbase_field_as_value (gchar *content, XBfield *field, XBfile *file)
 			 g_date_valid_dmy (day, month, year)) {
 			GDate *date = g_date_new_dmy (day, month, year);
 			/* Use default date convention */
-			val = value_new_int (datetime_g_to_serial (date, NULL));
+			val = value_new_int (go_date_g_to_serial (date, NULL));
 			g_date_free (date);
 		} else
 			val = value_new_string (s);
@@ -144,19 +144,19 @@ create_header (Sheet *sheet, XBfile *file)
 }
 
 void
-xbase_file_open (GOFileOpener const *fo, IOContext *io_context,
+xbase_file_open (GOFileOpener const *fo, GOIOContext *io_context,
                  WorkbookView *wb_view, GsfInput *input)
 {
 	Workbook  *wb;
 	XBfile	  *file;
 	XBrecord  *record;
 	Sheet	  *sheet = NULL;
-	ErrorInfo *open_error;
+	GOErrorInfo *open_error;
 	int rows = GNM_MAX_ROWS;
 	int pass;
 
 	if ((file = xbase_open (input, &open_error)) == NULL) {
-		gnumeric_io_error_info_set (io_context, error_info_new_str_with_details (
+		go_io_error_info_set (io_context, go_error_info_new_str_with_details (
 		                            _("Error while opening xbase file."),
 		                            open_error));
 		return;
