@@ -33,26 +33,19 @@
 #include <expr-impl.h>
 #include <sheet.h>
 #include <cell.h>
+#include <cell.h>
 
 /***************************************************************************/
 
 static GnmFuncHelp const help_sum[] = {
-    { GNM_FUNC_HELP_OLD,
-	N_("@FUNCTION=SUM\n"
-	   "@SYNTAX=SUM(value1, value2, ...)\n"
-
-	   "@DESCRIPTION="
-	   "SUM computes the sum of all the values and cells referenced "
-	   "in the argument list.\n\n"
-	   "* This function is Excel compatible."
-	   "\n"
-	   "@EXAMPLES=\n"
-	   "Let us assume that the cells A1, A2, ..., A5 contain numbers "
-	   "11, 15, 17, 21, and 43.  Then\n"
-	   "SUM(A1:A5) equals 107.")
-    },
-    { GNM_FUNC_HELP_SEEALSO, "AVERAGE,COUNT" },
-    { GNM_FUNC_HELP_END }
+	{ GNM_FUNC_HELP_NAME, N_("SUM:sum of the given values")},
+	{ GNM_FUNC_HELP_ARG, N_("values:a list of values to add")},
+	{ GNM_FUNC_HELP_DESCRIPTION, N_("SUM computes the sum of all the values and cells referenced in the argument list.")},
+	{ GNM_FUNC_HELP_EXCEL, N_("This function is Excel compatible.") },
+	{ GNM_FUNC_HELP_ODF, N_("This function is OpenFormula compatible.") },
+	{ GNM_FUNC_HELP_EXAMPLES, "=SUM(11,15,17,21,43)" },
+	{ GNM_FUNC_HELP_SEEALSO, "AVERAGE,COUNT"},
+	{ GNM_FUNC_HELP_END }
 };
 
 GnmValue *
@@ -69,21 +62,15 @@ gnumeric_sum (GnmFuncEvalInfo *ei, int argc, GnmExprConstPtr const *argv)
 /***************************************************************************/
 
 static GnmFuncHelp const help_product[] = {
-    { GNM_FUNC_HELP_OLD,
-	N_("@FUNCTION=PRODUCT\n"
-	   "@SYNTAX=PRODUCT(value1, value2, ...)\n"
-
-	   "@DESCRIPTION="
-	   "PRODUCT returns the product of all the values and cells "
-	   "referenced in the argument list.\n\n"
-	   "* This function is Excel compatible.  In particular, this means "
-	   "that if all cells are empty, the result will be 0.\n"
-	   "\n"
-	   "@EXAMPLES=\n"
-	   "PRODUCT(2,5,9) equals 90.")
-    },
-    { GNM_FUNC_HELP_SEEALSO, "SUM,COUNT,G_PRODUCT" },
-    { GNM_FUNC_HELP_END }
+	{ GNM_FUNC_HELP_NAME, N_("PRODUCT:product  of the given values")},
+	{ GNM_FUNC_HELP_ARG, N_("values:a list of values to multiply")},
+	{ GNM_FUNC_HELP_DESCRIPTION, N_("Product computes the product of all the values and cells referenced in the argument list.")},
+	{ GNM_FUNC_HELP_NOTE, N_("If all cells are empty, the result will be 0.") },
+	{ GNM_FUNC_HELP_EXCEL, N_("This function is Excel compatible.") },
+	{ GNM_FUNC_HELP_ODF, N_("This function is OpenFormula compatible.") },
+	{ GNM_FUNC_HELP_EXAMPLES, "=PRODUCT(2,5,9)" },
+	{ GNM_FUNC_HELP_SEEALSO, "SUM,COUNT,G_PRODUCT"},
+	{ GNM_FUNC_HELP_END }
 };
 
 static int
@@ -110,18 +97,11 @@ gnumeric_product (GnmFuncEvalInfo *ei, int argc, GnmExprConstPtr const *argv)
 /***************************************************************************/
 
 static GnmFuncHelp const help_gnumeric_version[] = {
-    { GNM_FUNC_HELP_OLD,
-	N_("@FUNCTION=GNUMERIC_VERSION\n"
-	   "@SYNTAX=GNUMERIC_VERSION()\n"
-
-	   "@DESCRIPTION="
-	   "GNUMERIC_VERSION returns the version of gnumeric as a string."
-
-	   "\n"
-	   "@EXAMPLES=\n"
-	   "GNUMERIC_VERSION().")
-    },
-    { GNM_FUNC_HELP_END }
+ 	{ GNM_FUNC_HELP_NAME, N_("GNUMERIC_VERSION:the current version of GNumeric")},
+	{ GNM_FUNC_HELP_DESCRIPTION, N_("GNUMERIC_VERSION returns the version of gnumeric as a string.")},
+	{ GNM_FUNC_HELP_EXAMPLES, "=GNUMERIC_VERSION()" },
+	{ GNM_FUNC_HELP_SEEALSO, ""},
+	{ GNM_FUNC_HELP_END }
 };
 
 static GnmValue *
@@ -162,7 +142,6 @@ gnumeric_table_link (GnmFuncEvalInfo *ei)
 static GnmValue *
 gnumeric_table (GnmFuncEvalInfo *ei, int argc, GnmExprConstPtr const *argv)
 {
-	GnmExpr const *arg;
 	GnmCell       *in[3], *x_iter, *y_iter;
 	GnmValue      *val[3], *res;
 	GnmCellPos     pos;
@@ -177,16 +156,20 @@ gnumeric_table (GnmFuncEvalInfo *ei, int argc, GnmExprConstPtr const *argv)
 		return value_new_error_REF (ei->pos);
 
 	for (x = 0; x < 2 ; x++) {
-		arg = (x < argc) ? argv[x] : NULL;
+		GnmExpr const *arg = argv[x];
 		val[x] = NULL;
 		if (NULL != arg && GNM_EXPR_GET_OPER (arg) == GNM_EXPR_OP_CELLREF) {
-			gnm_cellpos_init_cellref (&pos,
-				&arg->cellref.ref, &ei->pos->eval);
+			gnm_cellpos_init_cellref (&pos,	&arg->cellref.ref,
+						  &ei->pos->eval, ei->pos->sheet);
 			in[x] = sheet_cell_get (ei->pos->sheet, pos.col, pos.row);
 			if (NULL == in[x])
 				in[x] = sheet_cell_fetch (ei->pos->sheet, pos.col, pos.row);
-			else
+			else {
 				val[x] = in[x]->value;
+				if (gnm_cell_has_expr (in[x]) &&
+				    gnm_cell_expr_is_linked (in[x]))
+					dependent_unlink (&in[x]->base);
+			}
 		} else
 			in[x] = NULL;
 	}
@@ -246,6 +229,12 @@ gnumeric_table (GnmFuncEvalInfo *ei, int argc, GnmExprConstPtr const *argv)
 	}
 	if (NULL != in[2])
 		value_release (in[2]->value);
+	for (x = 0 ; x < 2 ; x++)
+		if (in[x] &&
+		    gnm_cell_has_expr (in[x]) &&
+		    !gnm_cell_expr_is_linked (in[x]))
+			dependent_link (&in[x]->base);
+
 	for (x = 0 ; x < 3 ; x++)
 		if (in[x]) {
 			dependent_queue_recalc (&in[x]->base);
@@ -262,50 +251,147 @@ gnumeric_table (GnmFuncEvalInfo *ei, int argc, GnmExprConstPtr const *argv)
 
 	return res;
 }
+
+/***************************************************************************/
+
+static GnmFuncHelp const help_if[] = {
+	{ GNM_FUNC_HELP_NAME, N_("IF:conditional expression.") },
+	{ GNM_FUNC_HELP_ARG, N_("cond:condition.") },
+	{ GNM_FUNC_HELP_ARG, N_("trueval:value to use if condition is true.") },
+	{ GNM_FUNC_HELP_ARG, N_("falseval:value to use if condition is false.") },
+	{ GNM_FUNC_HELP_DESCRIPTION, N_("This function first evaluates the condition.  If the result is true, it will then evaluate and return the second argument.  Otherwise, it will evaluate and return the last argument.") },
+        { GNM_FUNC_HELP_EXAMPLES, "=IF(1+2=3,\"x\",\"y\")" },
+	{ GNM_FUNC_HELP_SEEALSO, "AND,OR,XOR,NOT,IFERROR" },
+	{ GNM_FUNC_HELP_END }
+};
+
+GnmValue *
+gnumeric_if (GnmFuncEvalInfo *ei, GnmValue const * const *args)
+{
+	gboolean err;
+	int res = value_get_as_bool (args[0], &err) ? 1 : 2;
+
+	if (args[res])
+		return value_dup (args[res]);
+
+	if (ei->func_call->argc < res + 1)
+		/* arg-not-there: default to TRUE/FALSE.  */
+		return value_new_bool (res == 1);
+	else
+		/* arg blank: default to 0.  */
+		return value_new_int (0);
+}
+
+
+GnmValue *
+gnumeric_if2 (GnmFuncEvalInfo *ei, int argc, GnmExprConstPtr const *argv)
+{
+	gboolean err;
+	int i, branch;
+	GnmValue *args[3];
+	GnmValue *res;
+
+	g_return_val_if_fail (argc >= 1 && argc <= 3,
+			      value_new_error_VALUE (ei->pos));
+
+	/*
+	 * In this version of IF, we evaluate the arguments ourselves,
+	 * the call the regular IF.  However, arguments we do not need
+	 * we do not evaluate.
+	 *
+	 * IF is sometimes used to avoid expensive calculations.  Always
+	 * computing both branches destroys that intent.  See bug 326595.
+	 */
+
+	/* Evaluate condition.  */
+	res = gnm_expr_eval (argv[0], ei->pos, 0);
+	if (VALUE_IS_ERROR (res))
+		return res;
+	args[0] = res;
+
+	branch = value_get_as_bool (args[0], &err) ? 1 : 2;
+	for (i = 1; i <= 2; i++) {
+		GnmExprEvalFlags flags =
+			GNM_EXPR_EVAL_PERMIT_NON_SCALAR |
+			GNM_EXPR_EVAL_PERMIT_EMPTY;
+
+		args[i] = NULL;
+		if (i < argc && i == branch && !gnm_expr_is_empty (argv[i])) {
+			args[i] = gnm_expr_eval (argv[i], ei->pos, flags);
+			if (!args[i])
+				args[i] = value_new_empty ();
+		}
+	}
+
+	res = gnumeric_if (ei, (GnmValue const * const *)args);
+
+	for (i = 0; i <= 2; i++)
+		if (args[i])
+			value_release (args[i]);
+
+	return res;
+}
+
 /***************************************************************************/
 
 static GnmFuncGroup *math_group = NULL;
 static GnmFuncGroup *gnumeric_group = NULL;
+static GnmFuncGroup *logic_group = NULL;
 
 void
 func_builtin_init (void)
 {
+	const char *gname;
+	const char *textdomain = GETTEXT_PACKAGE;
+
 	static GnmFuncDescriptor const builtins [] = {
-		{	"sum",		NULL,	N_("number,number,"),
+		{	"sum",		NULL,
 			help_sum,	NULL,	gnumeric_sum,
 			NULL, NULL, NULL, GNM_FUNC_SIMPLE + GNM_FUNC_AUTO_FIRST,
 			GNM_FUNC_IMPL_STATUS_COMPLETE,
 			GNM_FUNC_TEST_STATUS_BASIC
 		},
-		{	"product",		NULL,	N_("number,number,"),
+		{	"product",		NULL,
 			help_product,	NULL,	gnumeric_product,
 			NULL, NULL, NULL, GNM_FUNC_SIMPLE,
 			GNM_FUNC_IMPL_STATUS_COMPLETE,
 			GNM_FUNC_TEST_STATUS_BASIC
 		},
-		{	"gnumeric_version",	"",	"",
+		{	"gnumeric_version",	"",
 			help_gnumeric_version,	gnumeric_version, NULL,
 			NULL, NULL, NULL, GNM_FUNC_SIMPLE,
 			GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC,
 			GNM_FUNC_TEST_STATUS_EXHAUSTIVE
 		},
-		{	"table",	"",	"",
+		{	"table",	"",
 			NULL,		NULL,	gnumeric_table,
 			gnumeric_table_link, NULL,
 			NULL, GNM_FUNC_SIMPLE + GNM_FUNC_INTERNAL,
 			GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC,
 			GNM_FUNC_TEST_STATUS_EXHAUSTIVE
 		},
+		{	"if", "b|EE",
+			help_if, gnumeric_if, NULL,
+			NULL, NULL, NULL,
+			GNM_FUNC_SIMPLE + GNM_FUNC_AUTO_SECOND,
+			GNM_FUNC_IMPL_STATUS_COMPLETE,
+			GNM_FUNC_TEST_STATUS_BASIC },
 		{ NULL }
 	};
 
-	math_group = gnm_func_group_fetch (N_("Mathematics"));
-	gnm_func_add (math_group, builtins + 0);
-	gnm_func_add (math_group, builtins + 1);
+	gname = N_("Mathematics");
+	math_group = gnm_func_group_fetch (gname, _(gname));
+	gnm_func_add (math_group, builtins + 0, textdomain);
+	gnm_func_add (math_group, builtins + 1, textdomain);
 
-	gnumeric_group = gnm_func_group_fetch (N_("Gnumeric"));
-	gnm_func_add (gnumeric_group, builtins + 2);
-	gnm_func_add (gnumeric_group, builtins + 3);
+	gname = N_("Gnumeric");
+	gnumeric_group = gnm_func_group_fetch (gname, _(gname));
+	gnm_func_add (gnumeric_group, builtins + 2, textdomain);
+	gnm_func_add (gnumeric_group, builtins + 3, textdomain);
+
+	gname = N_("Logic");
+	logic_group = gnm_func_group_fetch (gname, _(gname));
+	gnm_func_add (logic_group, builtins + 4, textdomain);
 }
 
 static void
@@ -322,4 +408,5 @@ func_builtin_shutdown (void)
 {
 	shutdown_cat (math_group);
 	shutdown_cat (gnumeric_group);
+	shutdown_cat (logic_group);
 }
